/*** 19990127 Frank Damgaard + others
 ***
 *** File: dump84.c
 ***
 *** dumps the content of a 16c84/16f84/24c16
 ***/


#ifdef __MSDOS__
#include "dosutil.h"
#else
#include <unistd.h>
#endif

#define EXTERN
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "prog84.h"
#include "progutil.h"


void dumpProg(WORD, WORD);
void dumpData(WORD, WORD);
void dumpCfg();

void dumphexProg(WORD, WORD);
void dumphexData(WORD);
void dumphexCfg(unsigned, unsigned);

int format;
pic_instr_t buf[16]; /* should be enough */

void dump_buf(pic_instr_t buf_count)
{
    int i;
    printf("D%d", buf_count);
    for(i=0; i<buf_count; i++)
	printf(" %04x", buf[i]);
    printf("\n");
}

void
start_cfg()
{
    SET(mclr, 0);
    SET(power, 1);
    SET(clock, 0);
    SET(data, 0);
    FLUSH;
    sleep(2);	/* for the serial prog. charge the capacitor */
    SET(mclr, 0);
    FLUSH;
    usleep(50000UL);/* for the serial prog. charge the capacitor */
    SET(mclr, 1);
    FLUSH;
    usleep(50000UL);
}

char *command;

void usage(char *txt)
{
   if (strlen(txt)>0) fprintf(stderr,"%s: %s\n",command,txt);
   fprintf(stderr, "usage: %s [options]\n"
                        " -p    dump pic program\n"
                        " -d    dump pic/eeprom data\n"
                        " -f    dump fuses\n"
                        " -i    dump id\n"
                        " -a    dump all of above\n"
			" -x    dump intel hex8 format (default)\n"
			" -y    dump intel hex16 format\n"
			" -t    dump pictool format\n"
			" -l NN dump only the first NN bytes \n"
			" -v    more verbose\n"
                        " -V    print version\n",
                        command);
        exit(1);
}

extern chip_info chips[];


int main(int argc, char *argv[])
{
    int prog=0,fuses=0,data=0,id=0;
    WORD last_addr=0;
    int opt;
    time_t now;
    char *end;
    
    set_progpath(argv[0]);
    verbose=0; format=IHX8M;
    command = argv[0]; opterr=0;

    while((opt=getopt(argc, argv, "afpdixythvVl:")) != -1) {
      switch(opt) {
	case 'a': fuses=data=id=prog=1; break;
	case 'f': fuses=1; break;
	case 'p': prog=1; break;
	case 'd': data=1; break;
	case 'i': id=1; break;
	case 'x': format=IHX8M; break;
	case 'y': format=IHX16; break;
	case 't': format=PICTOOL; break;
	case 'l': last_addr= (WORD) strtoul(optarg,&end,0);
		  if (last_addr> 0x1000) usage("length > 0x1000");
		break;
        case 'V':
	    printf("%s: Version %s\n",command,PROGVERS);
	    exit(0);
	    break;
	case 'v': 
		verbose ++;
		if(verbose > 1) { p84_verbose ++; i2c_debug++; } 
		break;
	case 'h':
		usage("");
		break;
	default:{
                    char txt[128];
                    sprintf(txt,"unknown option \"%c\"\n", optopt);
                    usage(txt);
	        }
      }
    }

    if ( ! (fuses||data||id||prog) ) {
#ifdef __MSDOS__
	exit(0);
#endif
	return 0;
    }
    progSetup();

#if 0
    { int ll;
     for ( ll=0; ll<=3; ll++) {
      fprintf(stderr," %s %4x %4x %4x\n",chips[ll].name,chips[ll].type,
		chips[ll].prog_size,chips[ll].eeprom_size);
     }
    }
#endif

    {   WORD sz;
	if (chip_p->type==EEPROM) sz=chip_p->eeprom_size;
	else sz=chip_p->prog_size;
	if (last_addr==0) last_addr=sz;
	if (last_addr>sz) {
		char txt[20];
		sprintf(txt,"length > 0x%x",sz);
		usage(txt);
	}
	if (verbose) fprintf(stderr,"dump last_addr=0x%04x %4x\n",last_addr,sz);
    }

    if (format != PICTOOL) {
	if (chip_p->type == PIC) {
	  if (prog) dumphexProg(0, last_addr);
	  if (id || fuses) dumphexCfg(id,fuses);
	  if (data) dumphexData(0);
	} else if (data) {
	  dumphexEEData(0,last_addr,format);
	}      
	printf(":00000001FF" HEX_EOL); /* end record */
    } else {
      if (chip_p->type != PIC) {
	fprintf(stderr,"Only Intel hex-file for eeprom's\n");
	fprintf(stdout,"#Only Intel hex-file for eeprom's\n");
      } else {
	time(&now);
	printf("# pictools format dump file\n# created by %s on %s",
	    argv[0], ctime(&now));
	if (id || fuses) dumpCfg();
	if (data) dumpData(0, 64);
	if (prog) dumpProg(0, last_addr);
      }
    }
    progShutdown();

#ifdef __MSDOS__
    exit(0);
#endif
    return 0;
}


void
dumpProg(WORD min_addr, WORD max_addr)
{
    int pic_addr=0;
    int buf_count;

    start_cfg();
    printf("A%04x\n", min_addr);

    for (pic_addr=0; pic_addr < min_addr; pic_addr++)
	incAddr();

    for (buf_count = 0 ; pic_addr < max_addr ; pic_addr++) {
	buf[buf_count ++] = readProg();
	if (buf_count >= 8) {
	    dump_buf(buf_count);
	    buf_count = 0;
	}
	incAddr();
    }
    if (buf_count)
	dump_buf(buf_count);
}


void
dumpData(WORD min_addr, WORD max_addr) {
    int pic_addr;
    int buf_count;

    start_cfg();
    printf("B%04x\n", min_addr);

    for (pic_addr=0; pic_addr < min_addr; pic_addr++)
	incAddr();

    for (buf_count = 0 ; pic_addr < max_addr ; pic_addr++) {
	buf[buf_count ++] = readData();
	if (buf_count >= 8) {
	    dump_buf(buf_count);
	    buf_count = 0;
	}
	incAddr();
    }
    if (buf_count)
	dump_buf(buf_count);
}

char *xtal_names[] = {
    "LP oscillator",
    "XT Oscillator",
    "HS Oscillator",
    "RC Oscillator"
};

void dumpCfg()
{
    unsigned int id[4], fuses;
    int i;

    start_cfg();

    printf("T84  # I can only understand '84s\n");

    /* dummy load configuration command to get to cfg memory */
    loadConfiguration(0x3FFF);
    for (i=0; i<7; i++) {
	if (i<4)
	    id[i] = readProg();
	incAddr();
    }
    fuses = readProg();

    printf("I%04x %04x %04x %04x  # id locations\n",
	id[0], id[1], id[2], id[3]);

    printf("%s\n%s\n%s\nC%d 0 # %s\n",
	fuses & 0x10 ?
	    "P1 # Code protection off" : "P0 # Code protection on",
	fuses & 0x08 ?
	    "U1 # Power-Up Timer Enabled" : "U0 # Power-Up Timer disabled",
	fuses & 0x04 ?
	    "W1 # Watchdog Enabled" : "W0 # Watchdog Disabled",
	fuses & 0x03, xtal_names[fuses & 3]
	);
}


void dumphexCfg(unsigned wid, unsigned wfuse)
{
    pic_instr_t fuses, id[4];
    int i;

    start_cfg();

    /* dummy load configuration command to get to cfg memory */
    loadConfiguration(0x3FFF);
    for (i=0; i<7; i++) {
	if (i<4)
	    id[i] = (pic_instr_t) readProg();
	incAddr();
    }
    fuses = (pic_instr_t) readProg();

    if (wid)   write_hex_record(stdout, 4, 0x2000, &id[0], format);
    if (wfuse) write_hex_record(stdout, 1, 0x2007, &fuses, format);
}

void dumphexProg(WORD min_addr, WORD max_addr)
{
    int pic_addr;
    int buf_count,beg_addr;

    start_cfg();

    for (pic_addr=0; pic_addr < min_addr; pic_addr++)
	incAddr();

    beg_addr=pic_addr;
    for (buf_count = 0 ; pic_addr < max_addr ; pic_addr++) {
	buf[buf_count ++] = readProg();
	if (buf_count >= 8) {
           write_hex_record(stdout, buf_count, beg_addr, buf, format);
	   beg_addr +=buf_count;
	   buf_count = 0;
	}
	incAddr();
    }
    if (buf_count)
           write_hex_record(stdout, buf_count, beg_addr, buf, format);
}


void dumphexData(WORD min_addr)
{
    int pic_addr;
    int buf_count,beg_addr;

    start_cfg();

    for (pic_addr=0; pic_addr < min_addr; pic_addr++)
	incAddr();

    beg_addr=pic_addr+0x2100;
    for (buf_count = 0 ; pic_addr < chip_p->eeprom_size ; pic_addr++) {
	buf[buf_count ++] = readData() & 0xff; /* data eeprom only 8 bit */
	if (buf_count >= 8) {
           write_hex_record(stdout, buf_count, beg_addr, buf, format);
	   beg_addr +=buf_count;
	   buf_count = 0;
	}
	incAddr();
    }
    if (buf_count)
           write_hex_record(stdout, buf_count, beg_addr, buf, format);
}

